//
// Simple_Node_OTAA
//
//  Demoprogramm, das einen TTN-Node per OTAA im Netz anmeldet (Join) und anschließend
//  alle TX_INTERVAL Sekunden den Text "Hello, world!" sendet.
//
//  Achtung: OTAA kann nicht mit Single-Channel-Gateways ohne Rückkanal (z.B. Dragino LG01)
//  verwendet werden, weil das Gateway die Join-Antwort nicht senden kann.
//
//  Boardeinstellung: Arduino Uno
//  Hardware: Arduino Uno mit Dragino LoRa Shield ( <-> Definition in lmic_pins)
//
//  Achtung: Das Programm erwartet die Datei node.h als Includedatei im selben Verzeichnis.
//  Darin sind die LoRaWAN-Parameter APPEUI, DEVEUI und APPKEY sowie der Devicename DEVICE
//  als Konstantendefinitionen enthalten.
//  Diese Datei muss erstellt werden bzw. wird zur Verfügung gestellt.
//  Beispielhafter Inhalt der Datei:
//
//        static const char* DEVICE = "device-otaa-00";
//        static const u1_t DEVEUI[8] = { 0x77, 0x66, ......, 0x00 };   // 8 bytes, LSB
//        static const u1_t APPEUI[8] = { 0x77, 0x66, ......, 0x00 };   // 8 bytes, LSB
//        static const u1_t APPKEY[16] = { 0x00, 0x11, ......, 0xFF };  // 16 bytes, MSB
//

#include <lmic.h>
#include <hal/hal.h>

#include "node.h"

void os_getArtEui (u1_t* buf) {
  memcpy(buf, APPEUI, 8);
}
void os_getDevEui (u1_t* buf) {
  memcpy(buf, DEVEUI, 8);
}
void os_getDevKey (u1_t* buf) {
  memcpy(buf, APPKEY, 16);
}

static uint8_t mydata[] = "Hello, world!";
static osjob_t sendjob;

// Schedule TX every this many seconds (might become longer due to duty cycle limitations).
const unsigned TX_INTERVAL = 10;

// Pin mapping
const lmic_pinmap lmic_pins = {
  .nss = 10,
  .rxtx = LMIC_UNUSED_PIN,
  .rst = 9,
  .dio = {2, 6, 7},
};

void onEvent (ev_t ev) {
  Serial.print(os_getTime());
  Serial.print(": ");
  switch (ev) {
    case EV_SCAN_TIMEOUT:
      Serial.println(F("EV_SCAN_TIMEOUT"));
      break;
    case EV_BEACON_FOUND:
      Serial.println(F("EV_BEACON_FOUND"));
      break;
    case EV_BEACON_MISSED:
      Serial.println(F("EV_BEACON_MISSED"));
      break;
    case EV_BEACON_TRACKED:
      Serial.println(F("EV_BEACON_TRACKED"));
      break;
    case EV_JOINING:
      Serial.println(F("EV_JOINING"));
      break;
    case EV_JOINED:
      Serial.println(F("EV_JOINED"));
#ifdef _LMIC_UTIL_H_
      {
        u4_t netid = 0;
        devaddr_t devaddr = 0;
        u1_t nwkKey[16];
        u1_t artKey[16];
        LMIC_getSessionKeys(&netid, &devaddr, nwkKey, artKey);
        Serial.print("netid: ");
        Serial.println(netid, DEC);
        Serial.print("devaddr: ");
        Serial.println(devaddr, HEX);
        Serial.print("artKey: ");
        for (int i = 0; i < sizeof(artKey); ++i) {
          Serial.print(artKey[i], HEX);
        }
        Serial.println("");
        Serial.print("nwkKey: ");
        for (int i = 0; i < sizeof(nwkKey); ++i) {
          Serial.print(nwkKey[i], HEX);
        }
        Serial.println("");
      }
#endif
      break;
    case EV_RFU1:
      Serial.println(F("EV_RFU1"));
      break;
    case EV_JOIN_FAILED:
      Serial.println(F("EV_JOIN_FAILED"));
      break;
    case EV_REJOIN_FAILED:
      Serial.println(F("EV_REJOIN_FAILED"));
      break;
      break;
    case EV_TXCOMPLETE:
      Serial.print(F("EV_TXCOMPLETE (includes waiting for RX windows); frequency = "));
      Serial.println(LMIC.freq);
      if (LMIC.txrxFlags & TXRX_ACK)
        Serial.println(F("Received ack"));
      if (LMIC.dataLen) {
        // data received in rx slot after tx
        Serial.print(F("Data Received: "));
        Serial.write(LMIC.frame + LMIC.dataBeg, LMIC.dataLen);
        Serial.println();
      }
      // Schedule next transmission
      os_setTimedCallback(&sendjob, os_getTime() + sec2osticks(TX_INTERVAL), do_send);
      break;
    case EV_LOST_TSYNC:
      Serial.println(F("EV_LOST_TSYNC"));
      break;
    case EV_RESET:
      Serial.println(F("EV_RESET"));
      break;
    case EV_RXCOMPLETE:
      // data received in ping slot
      Serial.println(F("EV_RXCOMPLETE"));
      break;
    case EV_LINK_DEAD:
      Serial.println(F("EV_LINK_DEAD"));
      break;
    case EV_LINK_ALIVE:
      Serial.println(F("EV_LINK_ALIVE"));
      break;
#ifdef _LMIC_UTIL_H_
    case EV_SCAN_FOUND:
      Serial.println(F("EV_SCAN_FOUND"));
      break;
    case EV_TXSTART:
      Serial.println(F("EV_TXSTART"));
      break;
#endif
    default:
      Serial.println(String(F("Unknown event: ")) + ev);
      break;
  }
}

void do_send(osjob_t* j) {
  // Check if there is not a current TX/RX job running
  if (LMIC.opmode & OP_TXRXPEND) {
    Serial.println(F("OP_TXRXPEND, not sending"));
  } else {
    // Prepare upstream data transmission at the next possible time.
    LMIC_setTxData2(1, mydata, sizeof(mydata) - 1, 0);
    Serial.println(F("Packet queued"));
  }
  // Next TX is scheduled after TX_COMPLETE event.
}

void setup() {
  Serial.begin(115200);
  Serial.println(String(F("\n\n Starting Simple_Node_OTAA for device ")) + DEVICE);

  // LMIC init
  os_init();

  // Reset the MAC state. Session and pending data transfers will be discarded.
  LMIC_reset();

  // Start job
  do_send(&sendjob);
}

void loop() {
  os_runloop_once();
}
